/*
*
* Copyright  2004, University of Washington, 
* Department of Computer Science and Engineering. All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions are met:
*
* 1. Redistributions of source code must retain the above copyright notice, 
* this list of conditions and the following disclaimer. 
* 
* 2. Redistributions in binary form must reproduce the above copyright notice,
* this list of conditions and the following disclaimer in the documentation 
* and/or other materials provided with the distribution.
*
* 3. Neither name of the University of Washington, Department of Computer 
* Science and Engineering nor the names of its contributors may be used to 
* endorse or promote products derived from this software without specific 
* prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR 
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
* SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
* CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
* OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
* 
*
*/

#include "moteconn.h"

// constructor
MoteConn::MoteConn(void)
{
	m_moteSFSocket = NULL;
}

// destructor
MoteConn::~MoteConn(void)
{
	WSACleanup();
}

/* establishes a connection to SerialForwarder 
* Parameters: 
ip: pointer to a string that contains the IP number
portNum: port that we're connecting to
* Returns: if successfully connected
*/
bool MoteConn::connectMoteSF(char* ip, int portNum)
{
	// initialize winsock	
	if(WSAStartup(MAKEWORD(2,2), &m_wsaData) != NO_ERROR)
	{
		cerr << "Error when attempting WSAStartup" << endl;
		return false;
	}

	// Create a socket
	m_moteSFSocket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	if(m_moteSFSocket == INVALID_SOCKET)
	{
		cerr << "Error opening a socket" << WSAGetLastError() << endl;
		return false;
	}

	// Connect to the Mote SerialForwarder
	sockaddr_in moteSerialForwarder;
	moteSerialForwarder.sin_family = AF_INET;
	moteSerialForwarder.sin_addr.s_addr = inet_addr(ip);
	moteSerialForwarder.sin_port = htons(portNum);

	if(connect(m_moteSFSocket, (SOCKADDR*) &moteSerialForwarder, sizeof(moteSerialForwarder)) == SOCKET_ERROR)
	{
		cerr << "Failed to connect to SerialFowarder" << WSAGetLastError() << endl;
		return false;
	}

	// Make sure that the correct protocol is being used when communicating with the serial forwarder
	char initBytes[2] = {'T', '!'};
	char platform[4] = {1, 0, 0, 0}; // set platform to avrmote
	char recvBuff[36];
	int bytesSent = send(m_moteSFSocket, initBytes, 2, 0);
	if(bytesSent != 2)
	{
		cerr << "Problem occured when trying to connect to SerialForward" << endl;
		return false;
	}

	// send platform
	bytesSent = send(m_moteSFSocket, platform, 4, 0);
	if(bytesSent != 4)
	{
		cerr << "Problem occured when trying to connect to SerialForward - sending platform" << endl;
		return false;
	}

	recv(m_moteSFSocket, recvBuff, 6, 0);
	if (recvBuff[0] != initBytes[0] || (recvBuff[1] & 0xff) < (initBytes[1] & 0xff))
	{
		cerr << "Problem with version of Serial Forwarder!!! Protocol does not match this implementation!" << endl;
		return false;
	}

	return true;
}

/* sends an array of bytes to serial forwarder
 * the array should follow TOSMsg conventions by starting with the header information
 * Parameters: 
	bytesToSend: pointer to packet array
	numBytes: number of bytes to send/length of bytesToSend
 * Returns: number of bytes that were sent
 */
int MoteConn::sendPacket(char* bytesToSend, int numBytes)
{
	if(numBytes > 255)
		return -1;

	char* sendBuffer = new char[numBytes + 1];

	// add the length of packet as preceding byte (serial forwarder protocol requirement)
	sendBuffer[0] = numBytes;
	for(int i = 0; i < numBytes; i++)
	{
		sendBuffer[i+1] = bytesToSend[i];
	}

	return (send(m_moteSFSocket, sendBuffer, numBytes+1, 0) - 1);
}

